@@ -1,12 +1,10 @@
 import re
 from dataclasses import dataclass
-from typing import Iterable, List, Tuple, Union
+from typing import Iterable, List, Tuple
 
-from rics.translation.offline.types import PlaceholdersTuple
+from rics.translation.offline.types import FormatType, PlaceholdersTuple
 from rics.utility.misc import tname
 
-FormatType = Union[str, "Format"]
-
 _REQUIRED_ELEMENT_RE = r"{(?P<required>\w+)}"
 _OPTIONAL_ELEMENT_REGEX = r"(?P<optional>\[(?P<left>.*?){(?P<optional_name>\w+)}(?P<right>.*?)\])"
 
@@ -14,12 +12,31 @@ _OPTIONAL_ELEMENT_REGEX = r"(?P<optional>\[(?P<left>.*?){(?P<optional_name>\w+)}
 class Format:
     """Format specification for translations strings.
 
-    Translator formats are similar to regular f-strings, with two important exceptions:
-    1. Positional arguments may not be used; ``'{}'`` is not accepted, correct form is ``'{key-name}'``.
-    2. Substrings surrounded by ``[]`` denote an optional element.
+    Translation formats are similar to regular f-strings, with two important exceptions:
+
+        1. Positional arguments (``'{}'``) may not be used; correct form is ``'{key-name}'``.
+        2. Substrings surrounded by ``[]`` denote an optional element.
 
     Args:
         fmt: A translation fstring.
+
+    Examples:
+        A format string with an optionl element.
+
+        >>> from rics.translation.offline import Format
+        >>> fmt = Format('{id}:{name}[, nice={is_nice}]')
+
+        The ``Format`` class when used directly only returns required placeholders by default..
+
+        >>> fmt.fstring(), fmt.fstring().format(id=0, name='Tarzan')
+        ('{id}:{name}', '0:Tarzan')
+
+        ..but the `placeholders` attribute can be used to retrieve all placeholders, required and optional:
+
+        >>> fmt.placeholders
+        ('id', 'name', 'is_nice')
+        >>> fmt.fstring(fmt.placeholders), fmt.fstring(fmt.placeholders).format(id=1, name='Morris', is_nice=True)
+        ('{id}:{name}, nice={is_nice}', '1:Morris, nice=True')
     """
 
     PLACEHOLDER_PATTERN: re.Pattern = re.compile(_OPTIONAL_ELEMENT_REGEX + "|" + _REQUIRED_ELEMENT_RE)
@@ -37,17 +54,14 @@ class Format:
         """Create a format string for the given placeholders.
 
         Args:
-            placeholders: Keys to keep. Passing None is equivalent to passing :attr:`required_placeholders`.
-            positional: If True, remove names to return a positional fstring.
+            placeholders: Keys to keep. Passing ``None`` is equivalent to passing :attr:`required_placeholders`.
+            positional: If ``True``, remove names to return a positional fstring.
 
         Returns:
             An fstring with optional elements removed unless included in `placeholders`.
 
         Raises:
             KeyError: If required placeholders are missing.
-
-        See Also:
-            :attr:`required_placeholders`
         """
         placeholders = placeholders or self.required_placeholders
         missing_required_placeholders = set(self.required_placeholders).difference(placeholders)
