@@ -4,14 +4,8 @@ from typing import Any, Collection, Dict, Generic, List, Union
 from rics._internal_support.types import NO_DEFAULT, NoDefault
 from rics.translation.offline._format import Format
 from rics.translation.offline._magic_dict import MagicDict
-from rics.translation.offline.types import (
-    IdType,
-    NameType,
-    PlaceholdersTuple,
-    PlaceholderTranslations,
-    SourceType,
-    TranslatedIds,
-)
+from rics.translation.offline.types import PlaceholdersTuple, PlaceholderTranslations, TranslatedIds
+from rics.translation.types import ID, IdType, NameType, SourceType
 from rics.utility.misc import tname
 
 
@@ -19,46 +13,41 @@ class FormatApplier(ABC, Generic[IdType, NameType, SourceType]):
     """Base class for application of ``Format`` specifications.
 
     Args:
-        placeholder_translations: Matrix of ID translation components returned by fetchers.
+        translations: Matrix of ID translation components returned by fetchers.
         default: Default values for each key in `placeholders`.
-        required_placeholders: Placeholder names which must be present in `default`. None=all.
+        required_placeholders: Placeholder names which must be present in `default`. All if ``None``.
 
     Raises:
         ValueError: If `default` is given and any placeholder names are missing.
-
-    See Also:
-        :class:`rics.translation.offline._format.Format`
     """
 
     def __init__(
         self,
-        placeholder_translations: PlaceholderTranslations,
+        translations: PlaceholderTranslations,
         default: Union[NoDefault, Dict[str, Any]] = NO_DEFAULT,
         required_placeholders: Collection[str] = None,
     ) -> None:
-        self._source = placeholder_translations.source
-        self._placeholder_names = placeholder_translations.placeholders
+        self._source = translations.source
+        self._placeholder_names = translations.placeholders
 
         if default is not NO_DEFAULT:
             required_defaults = set(
-                placeholder_translations.placeholders if required_placeholders is None else required_placeholders
+                translations.placeholders if required_placeholders is None else required_placeholders
             )
-            required_defaults.discard("id")
+            required_defaults.discard(ID)
             missing = required_defaults.difference(default)
             if missing:
                 raise ValueError(f"Placeholder names {sorted(missing)} not present in {default=}.")
 
         self._default = default
-        self._n_ids = len(placeholder_translations.records)
+        self._n_ids = len(translations.records)
 
-    def __call__(
-        self, fmt: Format, placeholders: PlaceholdersTuple = None, default_fmt: Format = None
-    ) -> MagicDict[IdType]:
+    def __call__(self, fmt: Format, placeholders: PlaceholdersTuple = None, default_fmt: Format = None) -> MagicDict:
         """Translate IDs.
 
         Args:
             fmt: Translation format to use.
-            placeholders: Placeholders to include in the formatted output. None=as many as possible.
+            placeholders: Placeholders to include in the formatted output. Use as many as possible if ``None``.
             default_fmt: Alternative format for default translation.
 
         Returns:
@@ -71,13 +60,18 @@ class FormatApplier(ABC, Generic[IdType, NameType, SourceType]):
         fstring = fmt.fstring(placeholders, self.positional)
         real_translations = self._apply(fstring, placeholders)
 
-        if default_fmt is None:
-            default_fstring = fstring
+        if self._default is NO_DEFAULT:
+            default_fstring = None
         else:
+            default_fmt = default_fmt or fmt
             placeholders = tuple(filter(self._placeholder_names.__contains__, default_fmt.placeholders))
-            default_fstring = default_fmt.fstring(placeholders, self.positional)
+            m = {**self._default, ID: "{}"} if ID in placeholders else self._default
+            default_fstring = default_fmt.fstring(placeholders).format(**m)
 
-        return MagicDict.make(real_translations, default_fstring, placeholders, self._default)
+        return MagicDict(
+            real_translations,
+            default_fstring,
+        )
 
     @abstractmethod
     def _apply(self, fstring: str, placeholders: PlaceholdersTuple) -> TranslatedIds:
@@ -95,7 +89,7 @@ class FormatApplier(ABC, Generic[IdType, NameType, SourceType]):
 
     @property
     def positional(self) -> bool:
-        """If True, names are stripped from fstring placeholders."""
+        """If ``True``, names are stripped from fstring placeholders."""
         return True
 
     @property
@@ -122,12 +116,12 @@ class DefaultFormatApplier(FormatApplier):
 
     def __init__(
         self,
-        placeholder_translations: PlaceholderTranslations,
+        translations: PlaceholderTranslations,
         default: Union[NoDefault, Dict[str, Any]] = NO_DEFAULT,
         required_placeholders: Collection[str] = None,
     ) -> None:
-        super().__init__(placeholder_translations, default, required_placeholders)
-        self._pht = placeholder_translations
+        super().__init__(translations, default, required_placeholders)
+        self._pht = translations
 
     def _apply(self, fstring: str, placeholders: PlaceholdersTuple) -> TranslatedIds:
         if self._placeholder_names == placeholders:
